/*
 * Copyright 2008 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.lucis.core;

import java.util.ArrayList;
import java.util.List;

import net.sf.derquinse.lucis.Item;
import net.sf.derquinse.lucis.Page;

import org.apache.lucene.search.Filter;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.ScoreDoc;
import org.apache.lucene.search.Sort;
import org.apache.lucene.search.TopDocCollector;
import org.apache.lucene.search.TopDocs;

/**
 * Abstract implementation for a search service.
 * @author Andres Rodriguez
 */
public abstract class LucisQuery<T> {
	private LucisQuery() {
	}
	
	TopDocs getTopDocs(LucisSearcher searcher, Query query, Filter filter, Sort sort, int hits) {
		final TopDocs docs;
		if (sort == null) {
			final TopDocCollector tdc = new TopDocCollector(hits);
			searcher.search(query, filter, tdc);
			docs = tdc.topDocs();
		} else {
			docs = searcher.search(query, filter, hits, sort);
		}
		return docs;
	}
	
	public abstract T perform(LucisSearcher searcher);

	public static <T> LucisQuery<Item<T>> first(final Query query, final Filter filter, final Sort sort, final DocMapper<T> mapper) {
		return new LucisQuery<Item<T>>() {
			public Item<T> perform(final LucisSearcher searcher) {
				final long t0 = System.currentTimeMillis();
				final TopDocs docs = getTopDocs(searcher, query, filter, sort, 1);
				final T item;
				final float score;
				if (docs.totalHits > 0) {
					final ScoreDoc sd = docs.scoreDocs[0];
					score = sd.score;
					item = mapper.map(sd.doc, score, searcher.doc(sd.doc));
				} else {
					score = 0.0f;
					item = null;
				}
				final long t1 = System.currentTimeMillis();
				return new Item<T>(docs.totalHits, score, t1 - t0, item);
			}
		};
	}

	public static <T> LucisQuery<Item<T>> first(final Query query, final Sort sort, final DocMapper<T> mapper) {
		return first(query, null, sort, mapper);
	}
	
	public static <T> LucisQuery<Item<T>> first(final Query query, final Filter filter, final DocMapper<T> mapper) {
		return first(query, filter, null, mapper);
	}
	
	public static <T> LucisQuery<Item<T>> first(final Query query, final DocMapper<T> mapper) {
		return first(query, null, null, mapper);
	}

	public static <T> LucisQuery<Page<T>> page(final Query query, final Filter filter, final Sort sort, final DocMapper<T> mapper, final int first, final int pageSize) {
		return new LucisQuery<Page<T>>() {
			public Page<T> perform(LucisSearcher searcher) {
				final int total = first + pageSize;
				final long t0 = System.currentTimeMillis();
				final TopDocs docs = getTopDocs(searcher, query, filter, sort, total);
				final List<T> items;
				final float score;
				if (docs.totalHits > first) {
					final int n = Math.min(total, docs.scoreDocs.length);
					items = new ArrayList<T>(n - first);
					score = docs.getMaxScore();
					for (int i = first; i < n; i++) {
						final ScoreDoc sd = docs.scoreDocs[i];
						final T item = mapper.map(sd.doc, score, searcher.doc(sd.doc));
						items.add(item);
					}
				} else {
					score = 0.0f;
					items = null;
				}
				final long t1 = System.currentTimeMillis();
				return new Page<T>(docs.totalHits, score, t1 - t0, first, items);
			}
		};
	}

	public static <T> LucisQuery<Page<T>> page(final Query query, final Sort sort, final DocMapper<T> mapper, final int first, final int pageSize) {
		return page(query, null, sort, mapper, first, pageSize);
	}

	public static <T> LucisQuery<Page<T>> page(final Query query, final Filter filter, final DocMapper<T> mapper, final int first, final int pageSize) {
		return page(query, filter, null, mapper, first, pageSize);
	}

	public static <T> LucisQuery<Page<T>> page(final Query query, final DocMapper<T> mapper, final int first, final int pageSize) {
		return page(query, null, null, mapper, first, pageSize);
	}
}
